const fs = require('fs')
const path = require('path')
const glob = require('glob')
const jsonPretty = require('json-stringify-pretty-compact')

const self = {
  // 解析路径（根据pwd）
  resolvePath(...args) {
    return path.resolve.apply(null, args)
  },

  // 连接路径
  joinPath(...args) {
    return path.join.apply(null, args)
  },

  // 获取目录名称
  getDirName(file, ...args) {
    if (path.isAbsolute(file)) {
      return path.dirname(file)
    }
    return path.dirname(path.resolve.apply(null, args.concat(file)))
  },

  // 获取文件的名称
  getFileName(pattern, options) {
    options = Object.assign({}, options)
    const { noExt } = options
    return self.matchFileSync(pattern, options).map((file) => {
      const name = file.replace(/(?:[^\\/]*?[\\/])*/g, '')
      return noExt && !name.startsWith('.') ? name.replace(/\..*$/g, '') : name
    })
  },

  // 获取目录短名称（不包含路径）
  getShortDirName(file, ...args) {
    return self.getDirName
      .apply(null, [file].concat(args))
      .replace(/(?:[^\\/]*?[\\/])*/g, '')
  },

  // 根据模式匹配路径
  matchFileSync(pattern, options) {
    return glob.sync(
      pattern.trim(),
      Object.assign(
        {
          nodir: true,
        },
        options
      )
    )
  },

  // 写文件
  writeFileSync(outputPath, data, json) {
    const output = self.resolvePath(outputPath)
    fs.writeFileSync(
      output,
      json ? jsonPretty(data, Object.assign({}, json)) : data
    )
    return output
  },
}

module.exports = self
